package org.rcisoft.bus.wmsgoodsapply.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.bus.wmsgoodsstock.entity.WmsGoodsStock;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.bus.wmsgoodsapply.dao.WmsGoodsApplyRepository;
import org.rcisoft.bus.wmsgoodsapply.entity.WmsGoodsApply;
import org.rcisoft.bus.wmsgoodsapply.service.WmsGoodsApplyService;
import org.rcisoft.bus.wmsgoodsstock.service.WmsGoodsStockService;

import org.rcisoft.sys.dictionary.dao.DictDataRepository;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class WmsGoodsApplyServiceImpl extends ServiceImpl<WmsGoodsApplyRepository,WmsGoodsApply>  implements WmsGoodsApplyService {
    @Autowired
    private DictDataRepository dictDataRepository;

    @Autowired
    private WmsGoodsStockService wmsGoodsStockRepositoryImpl;


    /**
    * 保存 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(WmsGoodsApply wmsGoodsApply){
        //增加操作
        //1.查库存表有没有对应的code  有则更新数量   没有则向库存表中插入
        //2.向日志表中添加对应的信息
        WmsGoodsStock wmsGoodsStock=new WmsGoodsStock();
        BeanUtils.copyProperties(wmsGoodsApply,wmsGoodsStock);
        List<WmsGoodsStock>wmsGoodsStockList=wmsGoodsStockRepositoryImpl.findAll(wmsGoodsStock);
        if (CollectionUtil.isEmpty(wmsGoodsStockList)){
            wmsGoodsStockRepositoryImpl.updateInCounts(wmsGoodsStock);
        }else{
            wmsGoodsStockRepositoryImpl.persist(wmsGoodsStock);
        }
        wmsGoodsApply.setIoType("0");
        wmsGoodsApply.setDealStatus("0");

        int line = baseMapper.insert(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }
    @Override
    public String AppCode() {
        // 获取当前时间
        Date date = new Date();
        // 设置日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        // 格式化日期
        String formattedDate = sdf.format(date);
        String codePrefix = "RX";
        int OrderCodeCount = baseMapper.getOrderCodeCount(codePrefix+formattedDate) + 1;
        String formattedCount = String.format("%03d",OrderCodeCount);
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 编号前缀
        String AppCode = codePrefix + formattedDate + formattedCount;
        return AppCode;
    }
    /**
    * 删除 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(WmsGoodsApply wmsGoodsApply){
        int line = baseMapper.realDelete(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(WmsGoodsApply wmsGoodsApply){
        wmsGoodsApply.setDeleted();
        int line = baseMapper.deleteById(wmsGoodsApply);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                wmsGoodsApply.getBusinessId()+"的商品申请表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(WmsGoodsApply wmsGoodsApply){
        int line = baseMapper.updateById(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 商品申请表
    * @param id
    * @return
    */
    @Override
    public WmsGoodsApply findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Override
    public IPage<WmsGoodsApply> findAllByPagination(CyPageInfo<WmsGoodsApply> paginationUtility,
        WmsGoodsApply wmsGoodsApply){
        return baseMapper.queryWmsGoodsApplysPaged(paginationUtility,wmsGoodsApply);

    }

    /**
    * 查询list 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Override
    public List<WmsGoodsApply> findAll(WmsGoodsApply wmsGoodsApply){
        return baseMapper.queryWmsGoodsApplys(wmsGoodsApply);

    }

    /**
    * 导出商品申请表
    * @return
    */
    @Override
    public List<WmsGoodsApply> export(WmsGoodsApply wmsGoodsApply) {
        List<WmsGoodsApply> wmsGoodsApplyList = baseMapper.queryWmsGoodsApplys(wmsGoodsApply);
        return wmsGoodsApplyList;
    }


    private String removeSymbol(String source) {
        String target = null;
        target = source.replaceAll("\\s*|\r|\n|\t","");
        if (target.contains(" ")) {
            target.replaceAll(" ","");
        }
        return target;
    }
    public Boolean isPositiveInteger(String figure) {
        // 如果输入为空，或者不是数字，则返回 false
        if (figure == null || figure.isEmpty()) {
            return false;
        }
        // 遍历字符串中的每个字符，检查是否都是数字
        for (char c : figure.toCharArray()) {
            if (!Character.isDigit(c)) {
                return false;
            }
        }
        // 将字符串转换为整数
        int num = Integer.parseInt(figure);
        // 如果大于 0 则为正整数，否则为非正整数
        return num > 0;
    }
}
